--[[
 AutoSummon module for MiniPet
 Version 5.1.201
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet autosummon Module" );

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true );
if( not PetAndMountDatabase ) then
    return;
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

if( not _G["MiniPet"] ) then
    return;
end--if

local AutoSummon = {
    displayname = L["Auto Summon"],
    settings = {
        [L["autosummon"]] = {
            name = L["Auto summon"],
            type = "toggle",
            arg = {
                DescriptionString = L["Toggles whether companions that are learned should be automatically summoned any time a warning would be given."],
                VariableName = "AutoSummon",
                Default = false,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,
        },
        [L["combatWait"]] = {
            name = L["Combat wait"],
            type = "toggle",
            arg = {
                DescriptionString = L["Toggles whether to wait until after combat is over before automatically summoning."],
                VariableName = "CombatWait",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,
        },
        [L["unsummonWhenStealthed"]] = {
            name = L["Unsummon when stealthed"],
            type = "toggle",
            arg = {
                DescriptionString = L["Toggles whether companions that are learned should be automatically unsummoned when stealthing/shadowmelding/etc."],
                VariableName = "UnsummonWhenStealthed",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,            
        },
    },
    events = {
        "WarnPlayer",
        "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained",
        "UNIT_SPELLCAST_CHANNEL_START",
        "UNIT_SPELLCAST_SENT",
        "UNIT_SPELLCAST_START",
        "UNIT_SPELLCAST_INTERRUPTED",
        "UNIT_SPELLCAST_STOP",
        "UNIT_SPELLCAST_FAILED",
        "UNIT_SPELLCAST_FAILED_QUIET",
        "UNIT_SPELLCAST_SUCCEEDED",
        "UNIT_SPELLCAST_CHANNEL_STOP",
        "LOOT_OPENED",
        "LOOT_CLOSED",
        "Update",
    },
    -- Module variables
    IsChanneling = false,
    IsLooting = false,
    DelaySummon = false,
    Counter = 0,
};

function AutoSummon:OnWarnPlayer( MiniPet )
    if( MiniPet.db.profile.AutoSummon and ( not PetAndMountDatabase:GetSummonedCompanionIndex( "CRITTER" ) ) and ( not IsMounted() ) ) then
        self.DelaySummon = true;
        self.Counter = 0;
    end--if
end--AutoSummon:OnWarnPlayer( MiniPet )

function AutoSummon:OnPlayerBuffGained( MiniPet, _, _, _, icon )
    if( MiniPet.db.profile.UnsummonWhenStealthed and
           ( ( icon == [[Interface\Icons\Ability_Stealth]] ) or ( icon == [[Interface\Icons\Ability_Ambush]] ) ) )  then
        MiniPet:DoCustomEvent( "DismissCompanion" );
    end--if
end--AutoSummon:OnPlayerBuffGained( MiniPet, _, _, _, icon )

function AutoSummon:OnUNIT_SPELLCAST_CHANNEL_START( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = true;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_CHANNEL_START( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_SENT( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = true;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_SENT( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_START( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = true;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_START( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_INTERRUPTED( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = false;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_INTERRUPTED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_STOP( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = false;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_STOP(( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_FAILED( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = false;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_FAILED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_FAILED_QUIET( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = false;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_FAILED_QUIET( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_SUCCEEDED( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = false;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_SUCCEEDED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_CHANNEL_STOP( MiniPet, unit )
    if( unit == "player" ) then
        self.IsChanneling = false;
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_CHANNEL_STOP( MiniPet, unit )

function AutoSummon:OnLOOT_OPENED( MiniPet )
    self.IsLooting = true;
end--AutoSummon:OnLOOT_OPENED( MiniPet )

function AutoSummon:OnLOOT_CLOSED( MiniPet )
    self.IsLooting = false;
end--AutoSummon:OnLOOT_CLOSED( MiniPet )

function AutoSummon:OnUpdate( MiniPet )
    if( self.DelaySummon and ( not self.IsChanneling ) and ( not self.IsLooting ) and
        ( not ( InCombatLockdown() and MiniPet.db.profile.CombatWait ) ) ) then
        self.Counter = self.Counter + 1;
        if( self.Counter > 2 ) then
            MiniPet:DoCustomEvent( "SummonCompanion" );
            self.DelaySummon = false;
            self.Counter = 0;
        end--if
    end--if
end--AutoSummon:OnUpdate( MiniPet )

_G["MiniPet"]:RegisterModule( "AutoSummon", AutoSummon, { "Companions", "Warning" } );
