--[[
 Companion module for MiniPet
 Version 5.1.201
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet" );

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true );
if( not PetAndMountDatabase ) then
    return;
end--if

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    return;
end--if

local ItemList = LibStub( "LibLordFarlander-ItemList-2.0", true );
if( not ItemList ) then
    return;
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true );
if( not TR ) then
    return;
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

if( not _G["MiniPet"] ) then
    return;
end--if

local Companions = {
    displayname = L["Companions"],
    events = {
		"BuildAdditionalOptions",
        "DismissCompanion",
        "GetAllCompanionsList",
        "SummonCompanion",
        "GetZoneList",
        "CheckItem",
        "AddBagItem",
        "AddInventoryItem",
        "ScanPlayer",
        "StartBagScan",
        "FinalizeBag",
        "ChooseRandom",
        "GetCurrentCompanion",
        "SetCurrentCompanion",
        "BuildZoneFavorites",
        "MouseClick",
        "COMPANION_UPDATE",
        "PLAYER_REGEN_DISABLED",
        "PLAYER_REGEN_ENABLED",
        "ZONE_CHANGED_NEW_AREA",
    },
    settings = {
        [L["boostNew"]] = {
            type = "toggle",
            order = 1,
            name = L["Boost new companions"],
            width = "double",
            arg = {
                DescriptionString = L["Toggles if new companions will be selected more often than old ones."],
                VariableName = "BoostNew",
                Default = false,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = "SetOption_ResetPets",
        },
        [L["boostNewTime"]] = {
            type = "range",
            order = 2,
            name = L["New time"],
            cmdHidden = true,
            min = 0.0,
            max = 31.0,
            bigStep = 1.0,
            desc = L["Set for how many days companions are concidered new."];
            get = function( info )
                      return info.handler.db.profile.BoostNewTime / 86400;
                  end,
            set = function( info, val )
                      info.handler.db.profile.BoostNewTime = val * 86400;
                  end,
        },
        [L["list"]] = {
            type = "execute",
            order = 3,
            cmdHidden = true,
            name = L["List all companions"],
            desc = L["Show a list with all of your companions."],
            func = "ListCompanions",
        },
        [L["refresh"]] = {
            type = "execute",
            order = 4,
            cmdHidden = true,
            name = L["Refresh companion list"],
            desc = L["Force a refresh of the list of all your companions."],
            func = "ResetCompanions",
        },    
    },
    required = true,
    -- Module variables
    CurrentCritterIndex  = nil,
    CurrentCompanion     = nil,
    PlayersCompanions    = { Zones = TR:GetTable( "default", TR:GetTable() ), List = {}, },
};

function Companions:BuildFavoritesOptions( MiniPet )
    local favoritesOptions = {
        type = "group",
        name = L["Favorites"],
        desc = L["Companions that should be picked more often if you have them."],
        handler = MiniPet,
        args = {
            favorite = {
                type = "select",
                order = 1,
                name = L["Favorites"],
                desc = L["Companions that should be picked more often if you have them."],
                width = "full",
                cmdHidden = true,
                values = function( info )
							 local handler = info.handler;				
							 local ui = handler.ui;
							
                             ui.Favorites = table.wipe( ui.Favorites );

                             local Favorites = ui.Favorites;
							 local ProfileFavorites = handler.db.profile.Favorites;
                             local firstItem = 0;
                             local used = TR:GetTable();
                             local companions = handler:GetAllCompanionsList();

                             for _, pet in pairs( companions ) do
                                 local id = pet:GetID();

                                 used[id] = true;
                                 table.insert( Favorites, id );
                                 if( type( ProfileFavorites[id] ) == "boolean" ) then
                                     ProfileFavorites[id] = 2.0;
                                 end--if
                                 if( ( type( ProfileFavorites[id] ) ~= "nil" ) and ( ProfileFavorites[id] == 1.0 ) ) then
                                     ProfileFavorites[id] = nil;
                                 end--if
                             end--for
                             for pet, howOften in pairs( ProfileFavorites ) do
                                 if( ( not used[pet] ) and ( ( pet < -1 ) and GetSpellInfo( pet * -1 ) or GetItemInfo( pet ) ) ) then
                                     table.insert( Favorites, pet );
                                 end--if
                             end--if
                             TR:ReturnTable( used );
                             table.sort( Favorites, function( a, b ) return ( ( a < 0 ) and GetSpellInfo( a * -1 ) or GetItemInfo( a ) ) < ( ( b < 0 ) and GetSpellInfo( b * -1 ) or GetItemInfo( b ) ) end );
                             Favorites = TR:GetTable();
                             for index, pet in pairs( ui.Favorites ) do
                                 local sName, sTexture;
                                 local sHowOften = L["default"];
                                 local howOften = ProfileFavorites[pet] or 1.0;

                                 if( pet < 0 ) then
                                     sName, _, sTexture, _, _, _, _, _, _ = GetSpellInfo( pet * -1 );
                                 else
                                     sName, _, _, _, _, _, _, _, _, sTexture = GetItemInfo( pet );
                                 end--if
                                 if( howOften == 0.0 ) then
                                     sHowOften = L["never"];
                                 elseif( howOften ~= 1.0 ) then
                                     sHowOften = L["%.1f times as often"]:format( howOften );
                                 end--if                                 
                                 if( sTexture ) then
                                     Favorites[index] = ("|T%s:0|t %s |cFFFF0000[%s]|r"):format( sTexture, sName, sHowOften );
                                 end--if
                                 if( firstItem == 0 ) then
                                     firstItem = index;
                                 end--if
                             end--for
                             if( ( not Favorites[ui.currentFavorite] ) or ( ui.currentFavorite == nil ) ) then
                                 ui.currentFavorite = firstItem;
                             end--if
                             TR:ReturnTable( table.wipe( companions ) );
                             return Favorites;
                         end,
                get = function( info )
                          return info.handler.ui.currentFavorite;
                      end,
                set = function( info, val )
                          info.handler.ui.currentFavorite = val;
                      end,
            },
            frequency = {
                type = "range",
                order = 2,
                name = L["Frequency"],
                cmdHidden = true,
                min = 0.0,
                max = 11.0,
                bigStep = 0.1,
                isPercent = true,
                desc = L["Set how often the companion gets picked.  2 is twice as often and 0 is never."];
                get = function( info )
                          return info.handler.db.profile.Favorites[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] or 1.0;
                      end,
                set = function( info, val )
                          if( val == 1.0 ) then
                              info.handler.db.profile.Favorites[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = nil;
                          else
                              info.handler.db.profile.Favorites[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = val;
                          end--if
                          info.handler:BuildZoneFavorites();
                      end,
            },
            [L["list"]] = {
                type = "execute",
                order = 4,
                guiHidden = true,
                name = L["List all favorite companions"],
                desc = L["Show a list with all of your favorite companions."],
                func = function( info )
                           info.handler:Print( L["List of your favorite companions:"] );
                           for pet, howOften in pairs( info.handler.db.profile.Favorites ) do
                               if( howOften ~= 1.0 ) then
                                   local link = ( pet > 0 ) and LibLordFarlander.GetItemLink( pet ) or GetSpellLink( pet * -1 );

                                   if( pet > 0 ) then
                                       sTexture = select( 10, GetItemInfo( pet ) );
                                   else
                                       sTexture = select( 3, GetSpellInfo( pet * -1 ) );
                                   end--if
                                   if( howOften == 0.0 ) then
                                       howOften = L["never"];
                                   else
                                       howOften = L["%.1f times as often"]:format( howOften );
                                   end--if

                                   if( sTexture ) then
                                       info.handler:Print( ("|T%s:0|t %s |cFFFF0000[%s]|r"):format( sTexture, link or tostring( pet ), howOften ) );
                                   else
                                       info.handler:Print( ("%s |cFFFF0000[%s]|r"):format( link or tostring( pet ), howOften ) );
                                   end--if
                               end--if
                           end--for
                       end,
            },
        },
    };
	
	return favoritesOptions;
end--Companions:BuildFavoritesOptions( MiniPet )

function Companions:BuildZoneFavoritesOptions( MiniPet )
    local zoneTemplate = {
        header = {
            type = "header",
            order = 1,
            width = "full",
            cmdHidden = true,
            name = function( info )
                       return LibLordFarlander.UnnamifyString( info[#info - 1] );
                   end,
        },
        minipet = {
            type = "select",
            order = 2,
            name = L["Companion"],
            desc = L["Companions that should be picked more often if you have them in the selected zone."],
            width = "full",
            cmdHidden = true,
            values = function( info )
                         info.handler.ui.Favorites = table.wipe( info.handler.ui.Favorites );

                         local Favorites = info.handler.ui.Favorites;
                         local used = TR:GetTable();
                         local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                         local companions = info.handler:GetAllCompanionsList();

                         for _, pet in pairs( info.handler:GetAllCompanionsList() ) do
                             local id = pet:GetID();

                             used[id] = true;
                             table.insert( Favorites, id );
                             if( type( info.handler.db.profile.Favorites[id] ) == "boolean" ) then
                                 info.handler.db.profile.Favorites[id] = 2.0;
                             end--if
                             if( ( type( info.handler.db.profile.Favorites[id] ) ~= "nil" ) and ( info.handler.db.profile.Favorites[id] == 1.0 ) ) then
                                 info.handler.db.profile.Favorites[id] = nil;
                             end--if                             
                         end--for
                         if( ZoneFavorite and ( type( ZoneFavorite ) ~= "table" ) ) then
                             info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )] = { [ZoneFavorite] = { frequency = 1.0, exclusive = true } };
                             ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                         end--if
                         if( not ZoneFavorite ) then
                             info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )] = TR:GetTable();
                             ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                         end--if
                         if( ZoneFavorite ) then
                             for pet, _ in pairs( ZoneFavorite ) do
                                 if( not used[pet] and ( ( pet < -1 ) and GetSpellInfo( pet * -1 ) or GetItemInfo( pet ) ) ) then
                                     table.insert( Favorites, pet );
                                     used[pet] = true;
                                 end--if
                             end--for
                         end--if
                         TR:ReturnTable( used );
                         table.sort( Favorites, function( a, b ) return ( ( a < 0 ) and GetSpellInfo( a * -1 ) or GetItemInfo( a ) ) < ( ( b < 0 ) and GetSpellInfo( b * -1 ) or GetItemInfo( b ) ) end );
                         Favorites = TR:GetTable();
                         for index, pet in pairs( info.handler.ui.Favorites ) do
                             local sName, sTexture;
                             local howOften = 1.0;
                             local sHowOften = L["default"]
                             local exclusive = false;

                             if( ZoneFavorite[pet] ) then
                                 howOften = ZoneFavorite[pet].frequency or 1.0;
                                 exclusive = ZoneFavorite[pet].exclusive;
                             end--if
                             if( pet < 0 ) then
                                 sName, _, sTexture, _, _, _, _, _, _ = GetSpellInfo( pet * -1 );
                             else
                                 sName, _, _, _, _, _, _, _, _, sTexture = GetItemInfo( pet );
                             end--if
                             if( sTexture ) then
                                 Favorites[index] = ("|T%s:0|t %s"):format( sTexture, sName );
                             end--if
                             if( howOften == 0.0 ) then
                                 sHowOften = L["never"];
                             elseif( howOften ~= 1.0 ) then
                                 sHowOften = L["%.1f times as often"]:format( howOften );
                             end--if
                             if( exclusive ) then
                                 sHowOften = string.format( "%s, %s", sHowOften, L["exclusive"] );
                             end--if
                             if( sTexture ) then
                                 Favorites[index] = ("|T%s:0|t %s |cFFFF0000[%s]|r"):format( sTexture, sName, sHowOften );
                             end--if
                         end--for
                         if( #Favorites == 0 ) then
                             info.handler.ui.currentFavorite = nil;
                             Favorites[0] = L["none"];
                         elseif( ( info.handler.ui.currentFavorite == nil ) or ( info.handler.ui.currentFavorite > #Favorites ) ) then
                             info.handler.ui.currentFavorite = 1;
                         end--if
                         TR:ReturnTable( table.wipe( companions ) );
                         return Favorites;
                     end,
            get = function( info )
                      return info.handler.ui.currentFavorite;
                  end,
            set = function( info, val )
                      info.handler.ui.currentFavorite = val;
                  end,
            },
            frequency = {
                type = "range",
                order = 3,
                name = L["Frequency"],
                cmdHidden = true,
                min = 0.0,
                max = 11.0,
                bigStep = 0.1,
                isPercent = true,
                desc = L["Set how often the companion gets picked.  2 is twice as often and 0 is never."];
                get = function( info )
                          local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];

                          if( ZoneFavorite and ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] ) then
                              return ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]].frequency or 1.0;
                          end--if
                          return 1.0;
                      end,
                set = function( info, val )
                          local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          local CurrentFavorite;

                          if( val == 1.0 ) then
                              val = nil;
                          end--if
                          if( not ZoneFavorite ) then
                              info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )] = TR:GetTable();
                              ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          end--if
                          CurrentFavorite = ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]];
                          if( not CurrentFavorite ) then
                              ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = { frequency = val };
                              CurrentFavorite = ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]];
                          else
                              CurrentFavorite.frequency = val;
                          end--if
                          if( not ( CurrentFavorite.frequency or CurrentFavorite.active or CurrentFavorite.exclusive ) ) then
                              ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = nil;
                          end--if
                          info.handler:BuildZoneFavorites();
                          info.handler:ChooseRandomCompanion();
                      end,
            },
            active = {
                type = "toggle",
                order = 4,
                name = L["Override favorites"],
                desc = L["Check this to override the favorites settings for this companion."];
                get = function( info )
                          local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];

                          if( ZoneFavorite and ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] ) then
                              return ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]].active or false;
                          end--if
                          return false;
                      end,
                set = function( info, val )
                          local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          local CurrentFavorite;

                          if( not ZoneFavorite ) then
                              info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )] = TR:GetTable();
                              ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          end--if
                          CurrentFavorite = ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]];
                          if( not CurrentFavorite ) then
                              ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = { active = val };
                              CurrentFavorite = ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]];
                          else
                              CurrentFavorite.active = val;
                          end--if
                          if( not ( CurrentFavorite.frequency or CurrentFavorite.active or CurrentFavorite.exclusive ) ) then
                              ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = nil;
                          end--if
                          info.handler:BuildZoneFavorites();
                          info.handler:ChooseRandomCompanion();
                      end,
            },
            exclusive = {
                type = "toggle",
                order = 5,
                name = L["Exclusive"],
                desc = L["Companions marked Exclusive will be the only ones picked from."];
                get = function( info )
                          local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          
                          if( ZoneFavorite and ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] ) then
                              return ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]].exclusive or false;
                          end--if
                          return false;
                      end,
                set = function( info, val )
                          local ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          local CurrentFavorite;

                          if( not ZoneFavorite ) then
                              info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )] = TR:GetTable();
                              ZoneFavorite = info.handler.db.profile.ZoneFavorites[LibLordFarlander.UnnamifyString( info[#info - 1] )];
                          end--if
                          CurrentFavorite = ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]];
                          if( not CurrentFavorite ) then
                              ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = { exclusive = val };
                              CurrentFavorite = ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]];
                          else
                              CurrentFavorite.exclusive = val;
                          end--if
                          if( not ( CurrentFavorite.frequency or CurrentFavorite.active or CurrentFavorite.exclusive ) ) then
                              ZoneFavorite[info.handler.ui.Favorites[info.handler.ui.currentFavorite]] = nil;
                          end--if
                          info.handler:BuildZoneFavorites();
                          info.handler:ChooseRandomCompanion();
                      end,
            },
    };
    local continentTemplate = {
    };
    local zoneOptions = {
        type = "group",
        name = L["Zone Favorites"],
        desc = L["Set a companion to use when in a certain zone."],
        handler = self,
        childGroups = "tree",
        args = {
            [L["list"]] = {
                type = "execute",
                order = -1,
                guiHidden = true,
                name = L["List all favorite companions"],
                desc = L["Show a list with all of your favorite companions."],
                func = function( info )
                           info.handler:Print( L["List of your favorite companions:"] );
                           for zone, pets in pairs( info.handler.db.profile.ZoneFavorites ) do
                               info.handler:Print( ("%s:"):format( zone ) );
                               for pet, settings in pairs( pets ) do
                                   local howOften = settings.frequency or 1.0;
                                   local sHowOften =  "default";
                                   local exclusive = settings.exclusive;

                                   if( ( howOften ~= 1.0 ) or exclusive ) then
                                       local link = ( pet > 0 ) and LibLordFarlander.GetItemLink( pet ) or GetSpellLink( pet * -1 );
    
                                       if( pet > 0 ) then
                                           sTexture = select( 10, GetItemInfo( pet ) );
                                       else
                                           sTexture = select( 3, GetSpellInfo( pet * -1 ) );
                                       end--if
                                       if( howOften == 0.0 ) then
                                           sHowOften = L["never"];
                                       elseif( howOften ~= 1.0 ) then
                                           sHowOften = L["%.1f times as often"]:format( howOften );
                                       end--if
                                       if( exclusive ) then
                                           sHowOften = ("%s, %s"):format( sHowOften, L["exclusive"] );
                                       end--if
                                       if( sTexture ) then
                                           info.handler:Print( ("|T%s:0|t %s |cFFFF0000[%s]|r"):format( sTexture, link or tostring( pet ), sHowOften ) );
                                       else
                                           info.handler:Print( ("%s |cFFFF0000[%s]|r"):format( link or tostring( pet ), sHowOften ) );
                                       end--if
                                   end--if
                               end--if
                           end--if
                       end,
            },
        },
    };

    for continentindex, continent in pairs( { GetMapContinents() } ) do
        zoneOptions.args[LibLordFarlander.NamifyString( continent )] = {
            type = "group",
            handler = MiniPet,
            name = continent,
            desc = L["Zones in %s"]:format( continent ),
            childGroups = "tree",
            args = TR:GetTable(),
        };

        local continentArgs = zoneOptions.args[LibLordFarlander.NamifyString( continent )].args;

        LibLordFarlander.CopyTable( continentTemplate, continentArgs );
        for index, zone in pairs( LibLordFarlander.GetMapZonesWithExtra( continentindex ) ) do
            continentArgs[LibLordFarlander.NamifyString( zone )] = {
                type = "group",
                handler = MiniPet,
                name = zone,
                desc = zone,
                args = zoneTemplate,
            };
        end--for
    end--for
	
	return zoneOptions;
end--Companions:BuildZoneFavoritesOptions( MiniPet )

function Companions:OnBuildAdditionalOptions( MiniPet )
    local options = {
        [L["Favorites"]] = self:BuildFavoritesOptions( MiniPet ),
        [L["Zone Favorites"]] = self:BuildZoneFavoritesOptions( MiniPet ),
    };
    
    return options;
end--Companions:OnBuildAdditionalOptions( MiniPet )

function Companions:OnCheckItem( MiniPet, ItemID, bag, slot )
    return itemID and PetAndMountDatabase:ItemInSet( itemID, "Critters" );
end--Companions:OnCheckItem( MiniPet, ItemID, bag, slot )

function Companions:OnScanPlayer( MiniPet, dirtyBags, FirstSeen )
    local returnVal = false;
    
    if( dirtyBags[-255] ) then
        local critters = PetAndMountDatabase:GetPlayersCompanions( "CRITTER" );
        local curTime = time();
        local spellList = self.PlayersCompanions.List[-255];
        local BuildSpell = ItemList.BuildSpell;
        
        if( not spellList ) then
            self.PlayersCompanions.List[-255] = TR:GetTable();
            spellList = self.PlayersCompanions.List[-255];
        end--if        
        for _, critter in pairs( critters ) do
		    local spellID = critter.spellID * -1;
			
            if( not FirstSeen[spellID] ) then
                FirstSeen[spellID] = curTime;
            end--if
            if( not spellList[spellID] ) then
			    spellList[spellID] = BuildSpell( critter.spellID );
                returnVal = true;
            end--if
        end--for
        TR:ReturnTable( critters );
    end--if
    return returnVal;
end--Companions:OnScanPlayer( MiniPet, dirtyBags, playerPetList, FirstSeen )

function Companions:OnStartBagScan( MiniPet, bag )
    self.PlayersCompanions.tempList = nil;
end--Companions:OnFinalizeBag( MiniPet )

function Companions:OnAddBagItem( MiniPet, bag, slot )
    local bagList = self.PlayersCompanions.tempList;
    
    if( not bagList ) then
        self.PlayersCompanions.tempList = TR:GetTable();
        bagList = self.PlayersCompanions.tempList;
    end--if
    ItemList.AddBagItemToList( bagList, bag, slot );
end--Companions:OnAddBagItem( MiniPet, bag, slot )

function Companions:OnAddInventoryItem( MiniPet, equipmentSlot )
    local bagList = self.PlayersCompanions.tempList;
    
    if( not bagList ) then
        self.PlayersCompanions.tempList = TR:GetTable();
        bagList = self.PlayersCompanions.tempList;
    end--if
    ItemList.AddInventoryItemToList( bagList, equipmentSlot );
end--Companions:OnAddInventoryItem( MiniPet, equipmentSlot )

function Companions:OnFinalizeBag( MiniPet, bag )
    local bagList = self.PlayersCompanions.tempList
    local PlayerList = self.PlayersCompanions.List;
    local changed = false;
    
    if( PlayerList[bag] ) then
        if( not LibLordFarlander.CompareTables( bagList, PlayerList[bag] ) ) then
            TR:ReturnTable( PlayerList[bag] );
            changed = true;
            PlayerList[bag] = bagList;
        else
            TR:ReturnTable( bagList );
        end--if
    elseif( bagList ) then
        changed = true;
        PlayerList[bag] = bagList;
    end--if
    self.PlayersCompanions.tempList = nil;
    return changed;
end--Companions:OnFinalizeBag( MiniPet, bag )

function Companions:OnCOMPANION_UPDATE( MiniPet, _, companionType )
    if( not companionType ) then
        MiniPet:DirtyBag( -255 );
    end--if
end--Companions:OnCOMPANION_UPDATE( MiniPet, _, companionType )

function Companions:OnPLAYER_REGEN_DISABLED( MiniPet )
    MiniPet:CombatStarted();
end--Companions:OnPLAYER_REGEN_DISABLED( MiniPet )

function Companions:OnPLAYER_REGEN_ENABLED( MiniPet )
    MiniPet:CombatEnded();
end--Companions:OnPLAYER_REGEN_ENABLED( MiniPet )

function Companions:OnZONE_CHANGED_NEW_AREA( MiniPet )
    MiniPet.ui.currentZone = GetRealZoneText();
    if( not LibLordFarlander.CompareTables( self.LastZoneList, self:OnGetZoneList() ) ) then
        MiniPet:ChooseRandomCompanion();
        self.LastZoneList = self:OnGetZoneList();
    end--if
    MiniPet:GetStateBasedOnModifiers();
end

function Companions:OnGetAllCompanionsList( MiniPet, indexed )
    local returnVal = TR:GetTable();

    for _, bag in pairs( self.PlayersCompanions.List ) do
        for _, item in pairs( bag ) do
            if( indexed ) then
                returnVal[item:GetID()] = item;
            else
                tinsert( returnVal, item );
            end--if
        end--for
    end--for
    return returnVal;
end--Companions:OnGetAllCompanionsList()

function Companions:OnBuildZoneFavorites( MiniPet )
    TR:ReturnTable( self.PlayersCompanions.Zones );
    self.PlayersCompanions.Zones = TR:GetTable( "default", TR:GetTable() );

    local Zones = self.PlayersCompanions.Zones;
    local companions = self:OnGetAllCompanionsList();
    local default = Zones.default;
    local profileFavorites = MiniPet.db.profile.Favorites;

    for zone, list in pairs( MiniPet.db.profile.ZoneFavorites ) do
        local favorites = TR:GetTable();
        local exclusives = TR:GetTable();

        for pet, settings in pairs( list ) do
            if( settings.active ) then
                favorites[pet] = settings.frequency or 1.0;
            end--if
            if( settings.exclusive ) then
                exclusives[pet] = settings.frequency or 1.0;
            end--if
        end--if
        if( LibLordFarlander.HasTableMembers( exclusives ) ) then
            TR:ReturnTable( favorites );
            Zones[zone] = exclusives;
        elseif( LibLordFarlander.HasTableMembers( favorites ) ) then
            TR:ReturnTable( exclusives );
            for _, item in pairs( companions ) do
                local id = item:GetID();

                if( not favorites[id] ) then
                    favorites[id] = profileFavorites[id] or 1.0;
                end--if
            end--if
            Zones[zone] = favorites;
        else
            TR:ReturnTable( exclusives );
            TR:ReturnTable( favorites );
        end--if
    end--for
    for  _, item in pairs( companions ) do
        local id = item:GetID();

        default[id] = profileFavorites[id] or 1.0;
    end--if
    self.LastZoneList = self:OnGetZoneList();
    TR:ReturnTable( table.wipe( companions ) );
end--Companions:OnBuildZoneFavorites( MiniPet )

function Companions:OnGetZoneList()
    return self.PlayersCompanions.Zones[GetRealZoneText()] or self.PlayersCompanions.Zones.default;
end--Companions:OnGetZoneList()

function Companions:OnSummonCompanion( MiniPet )
    if( self.CurrentCritterIndex ) then
        MiniPet:Print( L["Summoning %s"]:format( select( 2, GetCompanionInfo( "CRITTER", self.CurrentCritterIndex ) ) ) );
        CallCompanion( "CRITTER", self.CurrentCritterIndex );
        MiniPet:ChooseRandomCompanion();
        MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() );
    end--if
end--Companions:OnSummonCompanion()

function Companions:OnDismissCompanion()
    local SummonedCritterIndex = PetAndMountDatabase:GetSummonedCompanionIndex( "CRITTER" );
    
    if( SummonedCritterIndex )  then
        DismissCompanion( "CRITTER" );
    end--if
end--Companions:OnDismissCompanion()

function Companions:OnMouseClick( MiniPet, mouse, secure )
    if( ( btn == "RightButton" ) and PetAndMountDatabase:GetSummonedCompanionIndex( "CRITTER" ) ) then
        MiniPet:DismissCompanion();
        MiniPet:SetButton( self:GetStateBasedOnModifiers() );
    end--if
    return nil;
end--Companions:OnMouseClick( MiniPet, mouse )

function Companions:OnChooseRandom( MiniPet )
    local companions = self:OnGetAllCompanionsList( true );
    
    self.CurrentCompanion = MiniPet:GetRandomFromListWithWeighing( companions, self:OnGetZoneList(), 0.0 );
    TR:ReturnTable( table.wipe( companions ) );
end--Companions:OnChooseRandom( MiniPet )

function Companions:OnGetCurrentCompanion( MiniPet )
    self:OnSetCurrentCompanion( MiniPet, self.CurrentCompanion );
    return self.CurrentCompanion;
end--Companions:OnGetCurrentCompanion( MiniPet )

function Companions:OnSetCurrentCompanion( MiniPet, CurrentCompanion )
    if( CurrentCompanion and CurrentCompanion.spellID ) then
        self.CurrentCritterIndex = PetAndMountDatabase:FindCompanion( "CRITTER", CurrentCompanion.spellID );
    else
        self.CurrentCritterIndex = nil;
    end--if
end--Companions:OnSetCurrentCompanion( MiniPet, CurrentCompanion )

_G["MiniPet"]:RegisterModule( "Companions", Companions );
