--[[
 Companion enchant support module for MiniPet
 Version 5.1.201
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

if( not _G["MiniPet"] ) then
    return;
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet enchant item Module" );

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

local ItemList = LibStub( "LibLordFarlander-ItemList-2.0", true );
if( not ItemList ) then
    return;
end--if

local Enchant = {
    displayname = L["Companion Enchants"],
    settings = {
        [L["minipetItemModifier"]] = {
            name = L["Modifier key for modifier items"],
            arg = {
                VariableName = "CritterItemModifier",
                Default = "Shift",
            };
            desc = L["Changes the key that needs to be held down to pick a modifer item."],
            type = "select",
            values = { ["Disabled"] = L["Disabled"], ["Alt"] = L["Alt"], ["Control"] = L["Control"], ["Shift"] = L["Shift"], },
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,            
        },
    },
    events = {
        "AfterPlayerScan",
        "BuildAdditionalOptions",
        "FindNextCompanionEnchant",
        "ScanPlayer",
		"GetCurrentCompanion",
        "MouseClick",
    },
    -- Module variables    
    CurrentCompanionItem = nil,
    CurrentCompanionItemIndex = nil,
    CritterItemNames = {
        [35223] = L["Papa Hummel's Old-Fashioned Pet Biscuit"],
        [37460] = L["Rope Pet Leash"],
        [37431] = L["Fetch Ball"],
        [43352] = L["Pet Grooming Kit"],
        [43626] = L["Happy Pet Snack"],
        [44820] = L["Red Ribbon Pet Leash"],
    },
};

function Enchant:OnBuildAdditionalOptions( MiniPet )
    local options = {
        [L["Modifier Items"]] = {
			type = "group",
			name = L["Modifier Items"],
			desc = L["Allows you to set the order that modifier items are picked."],
			handler = MiniPet,
			args = {
				favorites = {
					type = "select",
					order = 1,
					name = L["Modifier Items"],
					desc = L["Allows you to set the order that modifier items are picked."],
					width = "full",
					cmdHidden = true,
					values = function( info )
								 local favorites = {};
                                 local Enchant = info.handler:GetModule( "Enchant" );

								 for _, item in pairs( info.handler.db.profile.enchantOrder ) do
									 local sName, _, _, _, _, _, _, _, _, sTexture = GetItemInfo( item );
									 
									 if( sTexture ) then
										 tinsert( favorites, ("|T%s:0|t %s"):format( sTexture, sName ) );
									 else
										 tinsert( favorites, Enchant.CritterItemNames[item] or item );
									 end--if
								 end--if
								 if( not info.handler.ui.currentModifierItem ) then
									 info.handler.ui.currentModifierItem = 1;
								 end--if
								 return favorites;
							 end,
					get = function( info )
							  return info.handler.ui.currentModifierItem;
						  end,
					set = function( info, val )
							  info.handler.ui.currentModifierItem = val;
						  end,
				},
				up = {
					type = "execute",
					order = 2,
					name = L["Move up"],
					desc = L["Moves the selected modifier up in the list."],
					cmdHidden = true,
					func = function( info )
							   if( info.handler.ui.currentModifierItem > 1 ) then
								   local olditem = info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem - 1];
								   
								   info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem - 1] = info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem];
								   info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem] = olditem;
								   info.handler.ui.currentModifierItem = info.handler.ui.currentModifierItem - 1;
								   info.handler:FindNextCompanionEnchant();
							   end--if
						   end,
				},
				down = {
					type = "execute",
					order = 2,
					name = L["Move down"],
					desc = L["Moves the selected modifier down in the list."],
					cmdHidden = true,
					func = function( info )
							   if( info.handler.ui.currentModifierItem < #info.handler.db.profile.enchantOrder ) then
								   local olditem = info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem + 1];

								   info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem + 1] = info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem];
								   info.handler.db.profile.enchantOrder[info.handler.ui.currentModifierItem] = olditem;
								   info.handler.ui.currentModifierItem = info.handler.ui.currentModifierItem + 1;
								   info.handler:FindNextCompanionEnchant();
							   end--if
						   end,
				},
			},
        },
    };
        
    return options;
end--Enchant:OnBuildAdditionalOptions( MiniPet )

function Enchant:OnScanPlayer( MiniPet )
    self.CurrentCompanionItem = nil;
end--Enchant:OnScanPlayer( MiniPet )

function Enchant:OnFindNextCompanionEnchant( MiniPet )
    local enchantOrder = MiniPet.db.profile.enchantOrder;
    local index = ( self.CurrentCompanionItemIndex or 0 ) + 1;
    local total = #enchantOrder;
    local count = 0;

    if( index > #enchantOrder ) then
        index = 1;
    end--if
    while( count < total ) do
        if( GetItemCount( enchantOrder[index] ) > 0 ) then
            self.CurrentCompanionItem = ItemList.BuildItem( enchantOrder[index] );
            self.CurrentCompanionItemIndex = index;
            return;
        end--if
        index = index + 1;
        if( index > #enchantOrder ) then
            index = 1;
        end--if
        count = count + 1;
    end--while
    self.CurrentCompanionItem = nil;
    self.CurrentCompanionItemIndex = nil;
end--Enchant:OnFindNextCompanionEnchant()

function Enchant:OnAfterPlayerScan( MiniPet )
    self.CurrentCompanionItemIndex = nil;
    self:OnFindNextCompanionEnchant( MiniPet );
end--Enchant:OnAfterPlayerScan( MiniPet )

function Enchant:OnGetCurrentCompanion( MiniPet )
    if( LibLordFarlander_UI.IsModifierDown( MiniPet.db.profile.CritterItemModifier ) and self.CurrentCompanionItem ) then
        return self.CurrentCompanionItem;
    end--if
	return nil;
end--Enchant:OnGetCurrentCompanion( MiniPet )

function Enchant:OnMouseClick( MiniPet, mouse, secure )
    if( ( ( secure and ( mouse == "RightButton" ) ) or ( not secure ) ) and LibLordFarlander_UI.IsModifierDown( MiniPet.db.profile.CritterItemModifier ) and self.CurrentCompanionItem ) then
        self:OnFindNextCompanionEnchant( MiniPet );
        return true;
    end--if
    return nil;
end--Enchant:OnMouseClick( MiniPet, mouse )

_G["MiniPet"]:RegisterModule( "Enchant", Enchant, { "Companions", "Quest" } );
