--[[
 Quest companion support module for MiniPet
 Version 5.1.201
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

if( not _G["MiniPet"] ) then
    return;
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet quest companion Module" );

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-2.0" ) );
    return;
end--if

local Quest = {
    displayname = L["Quest"],
    settings = {
        [L["forceQuestPet"]] = {
            type = "toggle",
            name = L["Force to pick a quest companion if availiable"],
            width = "double",                     
            arg = {
                DescriptionString = L["Toggles whether companions that are part of a quest are preferred instead of a random companion."],
                VariableName = "ForceQuestPet",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = "SetOption_ResetPets",                       
        },          
    },
    events = {
        "ScanPlayer",
        "GetCurrentCompanion",
        "UNIT_QUEST_LOG_CHANGED",
        "LibQuixote-2.0:Quest_Complete",
        "LibQuixote-2.0:Quest_Gained",
        "LibQuixote-2.0:Quest_Failed",
        "LibQuixote-2.0:Quest_Lost",        
    },
    -- Module variables    
    QuestCompanionQuest = nil,
    QuestCompanion = nil,
};

function Quest:OnScanPlayer( MiniPet, dirtyBags )
    if( self.QuestCompanionQuest and dirtyBags[self.QuestCompanionQuest.bag] ) then
        self.QuestCompanionQuest = nil;
    end--if
end--Quest:OnScanPlayer( MiniPet )

function Quest:OnCheckItem( MiniPet, ItemID, bag, slot )
    if( MiniPet.db.profile.ForceQuestPet ) then
        local questID = PetAndMountDatabase:ItemInSet( itemID, "Critters.Quest" );

        if( ( questID and self.HasQuestAndNotComplete( tonumber( questID ) ) ) or
                ( ( self.QuestCompanion == nil ) and PetAndMountDatabase:ItemInSet( itemID, "Critters.Children" ) ) ) then
            self.QuestCompanion = ItemList.BuildBagItem( bag, slot );
            if( questID ) then
                self.QuestCompanionQuest = tonumber( questID );
            else
                self.QuestCompanionQuest = true;
            end--if
        end--if
    end--if
end--Quest:OnCheckItem( MiniPet, ItemID, bag, slot )

function Quest:OnGetCurrentCompanion( MiniPet )
    if( MiniPet.db.profile.ForceQuestPet and self.QuestCompanion and self.QuestCompanionQuest ) then
        MiniPet:DoCustomEvent( "SetCurrentCompanion", self.QuestCompanion );
        return self.QuestCompanion;
    end--if
    return nil;
end--Quest:OnGetCurrentCompanion( MiniPet )

function Quest:OnQuestDone( MiniPet, _, _, uid )
    if( MiniPet.db.profile.ForceQuestPet and ( type( self.QuestCompanionQuest ) == "number" ) and ( self.QuestCompanionQuest == uid ) ) then
        self.QuestCompanionQuest = nil;
        self.QuestCompanion = nil;
        MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() );
    end--if
end--Quest:OnQuestDone( MiniPet, _, _, uid )

function Quest:OnQuest_Gained( MiniPet, _, _, uid )
    if( MiniPet.db.profile.ForceQuestPet and ( type( self.QuestCompanionQuest ) == "number" ) and ( self.QuestCompanionQuest == uid ) ) then
        for bag = 0, NUM_BAG_SLOTS do
            MiniPet.dirtyBags[bag] = true;
        end--for
        MiniPet.dirtyBags[-254] = true;
        MiniPet.dirtyBags[-255] = true;
        MiniPet:GetPlayersCompanions();
        MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() );
    end--if
end--Quest:OnQuest_Gained( MiniPet, _, _, uid )

function Quest:OnQuest_Complete( MiniPet, ... )
    self:OnQuestDone( MiniPet, ... );
end--Quest:OnQuest_Complete( MiniPet, ... )

function Quest:OnQuest_Failed( MiniPet, ... )
    self:OnQuestDone( MiniPet, ... );
end--Quest:OnQuest_Failed( MiniPet, ... )

function Quest:OnQuest_Lost( MiniPet, ... )
    self:OnQuestDone( MiniPet, ... );
end--Quest:OnQuest_Lost( MiniPet, ... )

function Quest:OnUNIT_QUEST_LOG_CHANGED( MiniPet, _, unit )
    if( not LibStub( "LibQuixote-2.0", true ) ) then
        if( ( unit == "player" ) and MiniPetdb.profile.ForceQuestPet and ( type( self.QuestCompanionQuest ) == "number" ) and self.QuestCompanionQuest ) then
            if( self.HasQuestAndNotComplete( self.QuestCompanionQuest ) ) then
                if( not self.QuestCompanion ) then
                    for bag = 0, NUM_BAG_SLOTS do
                        MiniPet.dirtyBags[bag] = true;
                    end--for
                    MiniPet.dirtyBags[-254] = true;
                    MiniPet.dirtyBags[-255] = true;
                    MiniPet:GetPlayersCompanions();
                    MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() );
                end--if
            else
                self.QuestCompanionQuest = nil;
                self.QuestCompanion = nil;
                MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() );
            end--if
        end--if
    end--if
end--Quest:OnUNIT_QUEST_LOG_CHANGED( MiniPet, _, unit )

function Quest.HasQuestAndNotComplete( questID )
    if( _G["Baldrick"] ) then
        return _G["Baldrick"].HasQuestAndNotComplete( questID );
    else
        local quixote = LibStub( "LibQuixote-2.0", true );

        if( quixote ) then
            local uid, _, _, _, _, _, complete, _, _, _ = quixote:GetQuestByUid( questID );
        
            return ( uid ~= nil) and ( not complete );
        else
            local numEntries, _ = GetNumQuestLogEntries();
        
            if( numEntries > 0 ) then
                for id = 1, numEntries do
                    local complete = select( 6, GetQuestLogTitle( id ) );
                    
                    if( ( not complete ) and ( LibLordFarlander.GetItemIDFromLink( GetQuestLink( id ) ) == questID ) ) then
                        return true;
                    end--if
                end--for
            end--if
        end--if
    end--if
    return false;
end--Quest.HasQuestAndNotComplete( questID )

_G["MiniPet"]:RegisterModule( "Quest", Quest, "Companions" );
