--[[
 Reagented Companion module for MiniPet
 Version 5.1.201
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet reagented Module" );

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true );
if( not PetAndMountDatabase ) then
    return;
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

if( not _G["MiniPet"] ) then
    return;
end--if

local Reagented = {
    displayname = L["Reagented"],
    settings = {
        [L["reagentCheck"]] = {
            type = "toggle",
            name = L["Check for reagented companions that are missing their reagent"],
            width = "double",                     
            arg = {
                DescriptionString = L["Toggles whether companions that require a reagent will be suggested when you have none of the required reagent."],
                VariableName = "ReagentCheck",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = "SetOption_ResetPets",                       
        },        
    },
    events = {
        "VerifyRandom",
    },
    -- Module variables
    CurrentCritterIndex = nil,
};

function Reagented:OnVerifyRandom( MiniPet, itemID )
    if( MiniPet.db.profile.ReagentCheck ) then
        reagented = PetAndMountDatabase:ItemInSet( itemID, "Critters.Reagented" );
        return ( not reagented ) or ( GetItemCount( reagented ) > 0 );        
    end--if
    return true;
end--Reagented:OnVerifyRandom( MiniPet, itemID )

_G["MiniPet"]:RegisterModule( "Reagented", Reagented, "Companions" );
